package com.persagy.apm.energy.report.monthly.detail.business.service.impl;

import com.persagy.apm.energy.report.monthly.detail.business.service.IBusinessReportCostInfoService;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.CostItemVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.CostVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.GroupInfo;
import com.persagy.apm.energy.report.monthly.detail.common.service.impl.ReportCostInfoServiceImpl;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 商业类报告费用service类
 *
 * @author lixing
 * @version V1.0 2021/5/30 7:30 下午
 **/
@Service
public class BusinessReportCostInfoServiceImpl extends ReportCostInfoServiceImpl
        implements IBusinessReportCostInfoService {
    @Autowired
    private ISaasWebService saasWebService;
    @Autowired
    private IReportOutlineService reportOutlineService;

    @Override
    public CostVO getOpenCost(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList) {

        Date reportMonth = reportOutline.getReportMonth();

        List<String> projectIds = reportOutlineService.queryRelatedProjectIds(reportOutline);

        return getCostInfoByProjectIds(groupIdList, functionIdList, reportMonth, projectIds);
    }

    @Override
    public CostVO getCompareCost(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList) {

        Date reportMonth = reportOutline.getReportMonth();

        List<String> projectIds = reportOutlineService.queryRelatedProjectIds(reportOutline);

        List<String> comparableProjects = saasWebService.getComparableProjects(projectIds, reportMonth);

        return getCostInfoByProjectIds(groupIdList, functionIdList, reportMonth, comparableProjects);
    }

    /**
     * 根据项目id获取费用信息
     *
     * @param groupIdList    费用信息点分组列表
     * @param functionIdList 费用信息点列表
     * @param reportMonth    报告月份
     * @param projectIds     项目id列表
     * @return 费用信息
     * @author lixing
     * @version V1.0 2021/5/30 11:41 下午
     */
    private CostVO getCostInfoByProjectIds(List<String> groupIdList, List<String> functionIdList, Date reportMonth, List<String> projectIds) {
        CostVO costVO = new CostVO();
        List<GroupInfo<CostItemVO>> costGroupInfos = getCostGroupInfos(groupIdList, reportMonth, projectIds);
        costVO.setGroups(costGroupInfos);

        List<CostItemVO> costItemInfos = getCostItemInfos(functionIdList, reportMonth, projectIds);
        costVO.setItems(costItemInfos);

        List<CostItemVO> paragraphSums = new ArrayList<>();

        if (!CollectionUtils.isEmpty(costGroupInfos)) {
            paragraphSums.addAll(
                    costGroupInfos.stream().map(GroupInfo::getSummary).collect(Collectors.toList()));
        }
        if (!CollectionUtils.isEmpty(costItemInfos)) {
            paragraphSums.addAll(costItemInfos);
        }
        costVO.setSummary(getCostSum(paragraphSums));
        return costVO;
    }
}
